package org.openxma.demo.customer;

import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.List;
import org.openxma.demo.customer.dto.CustomerContactReport;
import org.openxma.demo.customer.dto.CustomerView;
import org.openxma.demo.customer.model.Customer;
import org.openxma.dsl.platform.assembler.DtoAssembler;
import org.openxma.dsl.platform.assembler.DtoAssemblerFactory;
import org.openxma.dsl.platform.assembler.DtoAssemblerRegistry;
import org.openxma.dsl.platform.converter.Converter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;

public class DtoAssemblerFactoryCustomerGen implements DtoAssemblerFactory {

	private List<DtoAssembler<?>> dtoAssemblers;

	@Autowired
	private DtoAssemblerRegistry dtoAssemblerRegistry;
	
	@Autowired
	private Converter converter;

	public DtoAssemblerFactoryCustomerGen() {
		dtoAssemblers = new ArrayList<DtoAssembler<?>>(2);
		initDtoAssemblers();
	}


	public List<DtoAssembler<?>> getDtoAssemblers() {
		return dtoAssemblers;
	}

	protected DtoAssemblerRegistry getDtoAssemblerRegistry() {
		return dtoAssemblerRegistry;
	}
	
	protected Converter getConverter() {
		return converter;
	}

	private void initDtoAssemblers() {
		dtoAssemblers.add(createDtoAssemblerCustomerContactReport());
		dtoAssemblers.add(createDtoAssemblerCustomerView());
	}


	private DtoAssembler<CustomerContactReport> createDtoAssemblerCustomerContactReport() {
		return new DtoAssembler<CustomerContactReport>() {
			public CustomerContactReport mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				if (sourceEntity instanceof Customer) {
					Customer customer = (Customer) sourceEntity;
					CustomerContactReport result = mapFromCustomerToCustomerContactReport(customer);
					return result;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public CustomerContactReport mapFromEntity(Object sourceEntity, CustomerContactReport targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
				if (sourceEntity instanceof Customer) {
					Customer customer = (Customer) sourceEntity;
					mapFromCustomerToCustomerContactReport(customer, targetDto);
					return targetDto;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public <T> T mapTo(CustomerContactReport sourceDto, Class<T> target) {
				if (CustomerContactReport.class.equals(target)) {
					CustomerContactReport result = new CustomerContactReport();
					mapFromCustomerContactReportToCustomerContactReport(sourceDto, result);
					return target.cast(result);
				}
				else if (Customer.class.equals(target)) {
					return target.cast(mapFromCustomerContactReportToCustomer(sourceDto));
				}
				else {
					throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
				}
			}
			
			public void mapTo(CustomerContactReport sourceDto, Object targetObject) {
				if (targetObject instanceof CustomerContactReport) {
					mapFromCustomerContactReportToCustomerContactReport(sourceDto, (CustomerContactReport) targetObject);
				}
				else if (targetObject instanceof Customer) {
					mapFromCustomerContactReportToCustomer(sourceDto, (Customer)targetObject);
				}			   
				else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}
	
	protected void mapFromCustomerContactReportToCustomerContactReport(CustomerContactReport sourceDto, CustomerContactReport customerContactReport) {
		customerContactReport.setFirstName(sourceDto.getFirstName());
		customerContactReport.setLastName(sourceDto.getLastName());
		customerContactReport.setEmailAddress(sourceDto.getEmailAddress());
	}
	
	protected Customer mapFromCustomerContactReportToCustomer(CustomerContactReport sourceDto) {
		return mapFromCustomerContactReportToCustomer(sourceDto, MapperContextHolder.createEntity(Customer.class));
	}
	
	protected Customer mapFromCustomerContactReportToCustomer(CustomerContactReport sourceDto, Customer customer) {
		if (MapperContextHolder.getContext().containsKey(sourceDto)) {
			return (Customer) MapperContextHolder.getContext().get(sourceDto);
		}
		MapperContextHolder.getContext().put(sourceDto, customer);
		customer.setFirstName(sourceDto.getFirstName());
		customer.setLastName(sourceDto.getLastName());
		customer.setEmailAddress(sourceDto.getEmailAddress());
		return customer;
	}
	
	protected CustomerContactReport mapFromCustomerToCustomerContactReport(Customer customer) {
		if (MapperContextHolder.getContext().containsKey(customer)) {
			return (CustomerContactReport) MapperContextHolder.getContext().get(customer);
		}
		CustomerContactReport customerContactReport = new CustomerContactReport();
		MapperContextHolder.getContext().put(customer, customerContactReport);
		customerContactReport.setFirstName(customer.getFirstName());
		customerContactReport.setLastName(customer.getLastName());
		customerContactReport.setEmailAddress(customer.getEmailAddress());
		return customerContactReport;
	}
	
	protected CustomerContactReport mapFromCustomerToCustomerContactReport(Customer customer, CustomerContactReport customerContactReport) {
		if (MapperContextHolder.getContext().containsKey(customer)) {
			return (CustomerContactReport) MapperContextHolder.getContext().get(customer);
		}
		MapperContextHolder.getContext().put(customer, customerContactReport);
		customerContactReport.setFirstName(customer.getFirstName());
		customerContactReport.setLastName(customer.getLastName());
		customerContactReport.setEmailAddress(customer.getEmailAddress());
		return customerContactReport;
	}
	
	
	
	private DtoAssembler<CustomerView> createDtoAssemblerCustomerView() {
		return new DtoAssembler<CustomerView>() {
			public CustomerView mapFromEntity(Object sourceEntity) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				if (sourceEntity instanceof Customer) {
					Customer customer = (Customer) sourceEntity;
					CustomerView result = mapFromCustomerToCustomerView(customer);
					return result;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public CustomerView mapFromEntity(Object sourceEntity, CustomerView targetDto) {
				Assert.notNull(sourceEntity, "parameter 'sourceEntity' must not be null");
				Assert.notNull(targetDto, "parameter 'targetDto' must not be null");
				if (sourceEntity instanceof Customer) {
					Customer customer = (Customer) sourceEntity;
					mapFromCustomerToCustomerView(customer, targetDto);
					return targetDto;
				}
				throw new IllegalArgumentException("mapping from object '"+sourceEntity+"' is not supported");
					
			}
			
			public <T> T mapTo(CustomerView sourceDto, Class<T> target) {
				if (CustomerView.class.equals(target)) {
					CustomerView result = new CustomerView();
					mapFromCustomerViewToCustomerView(sourceDto, result);
					return target.cast(result);
				}
				else if (Customer.class.equals(target)) {
					return target.cast(mapFromCustomerViewToCustomer(sourceDto));
				}
				else {
					throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
				}
			}
			
			public void mapTo(CustomerView sourceDto, Object targetObject) {
				if (targetObject instanceof CustomerView) {
					mapFromCustomerViewToCustomerView(sourceDto, (CustomerView) targetObject);
				}
				else if (targetObject instanceof Customer) {
					mapFromCustomerViewToCustomer(sourceDto, (Customer)targetObject);
				}			   
				else if (targetObject != null) {
					throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
				}
			}
		};
	}
	
	protected void mapFromCustomerViewToCustomerView(CustomerView sourceDto, CustomerView customerView) {
		customerView.setOid(sourceDto.getOid());
		customerView.setVersion(sourceDto.getVersion());
		customerView.setFirstName(sourceDto.getFirstName());
		customerView.setLastName(sourceDto.getLastName());
		customerView.setEmailAddress(sourceDto.getEmailAddress());
		customerView.setBirthDate(sourceDto.getBirthDate());
		customerView.setInvoiceAddress(sourceDto.getInvoiceAddress());
		customerView.setDeliveryAddress(sourceDto.getDeliveryAddress());
		customerView.setVip(sourceDto.getVip());
		customerView.setCategory(sourceDto.getCategory());
		customerView.setBuyLimit(sourceDto.getBuyLimit());
	}
	
	protected Customer mapFromCustomerViewToCustomer(CustomerView sourceDto) {
		return mapFromCustomerViewToCustomer(sourceDto, MapperContextHolder.createEntity(Customer.class));
	}
	
	protected Customer mapFromCustomerViewToCustomer(CustomerView sourceDto, Customer customer) {
		if (MapperContextHolder.getContext().containsKey(sourceDto)) {
			return (Customer) MapperContextHolder.getContext().get(sourceDto);
		}
		MapperContextHolder.getContext().put(sourceDto, customer);
		if (sourceDto.getOid() != null) {
			customer.setOid(sourceDto.getOid());
		}
		customer.setVersion(sourceDto.getVersion());
		customer.setFirstName(sourceDto.getFirstName());
		customer.setLastName(sourceDto.getLastName());
		customer.setEmailAddress(sourceDto.getEmailAddress());
		customer.setBirthDate(sourceDto.getBirthDate());
		customer.setInvoiceAddress(sourceDto.getInvoiceAddress());
		customer.setDeliveryAddress(sourceDto.getDeliveryAddress());
		customer.setVip(sourceDto.getVip());
		customer.setCategory(sourceDto.getCategory());
		customer.setBuyLimit(sourceDto.getBuyLimit());
		return customer;
	}
	
	protected CustomerView mapFromCustomerToCustomerView(Customer customer) {
		if (MapperContextHolder.getContext().containsKey(customer)) {
			return (CustomerView) MapperContextHolder.getContext().get(customer);
		}
		CustomerView customerView = new CustomerView();
		MapperContextHolder.getContext().put(customer, customerView);
		customerView.setOid(customer.getOid());
		customerView.setVersion(customer.getVersion());
		customerView.setFirstName(customer.getFirstName());
		customerView.setLastName(customer.getLastName());
		customerView.setEmailAddress(customer.getEmailAddress());
		customerView.setBirthDate(customer.getBirthDate());
		customerView.setInvoiceAddress(customer.getInvoiceAddress());
		customerView.setDeliveryAddress(customer.getDeliveryAddress());
		customerView.setVip(customer.getVip());
		customerView.setCategory(customer.getCategory());
		customerView.setBuyLimit(customer.getBuyLimit());
		return customerView;
	}
	
	protected CustomerView mapFromCustomerToCustomerView(Customer customer, CustomerView customerView) {
		if (MapperContextHolder.getContext().containsKey(customer)) {
			return (CustomerView) MapperContextHolder.getContext().get(customer);
		}
		MapperContextHolder.getContext().put(customer, customerView);
		customerView.setOid(customer.getOid());
		customerView.setVersion(customer.getVersion());
		customerView.setFirstName(customer.getFirstName());
		customerView.setLastName(customer.getLastName());
		customerView.setEmailAddress(customer.getEmailAddress());
		customerView.setBirthDate(customer.getBirthDate());
		customerView.setInvoiceAddress(customer.getInvoiceAddress());
		customerView.setDeliveryAddress(customer.getDeliveryAddress());
		customerView.setVip(customer.getVip());
		customerView.setCategory(customer.getCategory());
		customerView.setBuyLimit(customer.getBuyLimit());
		return customerView;
	}
	
	
	
}
