package org.openxma.demo.customer.dao;

import org.openxma.dsl.platform.dao.*;
import java.util.Collection;
import java.util.Date;
import org.openxma.demo.customer.Address;
import org.openxma.demo.customer.dao.CustomerDao;
import org.openxma.demo.customer.dto.CustomerContactReport;
import org.openxma.demo.customer.model.Customer;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity CustomerDao.
 * @see CustomerDao
 */
public interface CustomerDaoGen extends GenericDao<Customer,String>, EntityFactory<Customer> {
	/**
	 * Query name constants
	 */
	static final String FIND_ALL_CUSTOMERS = "Customer.FindAllCustomers";
	static final String FIND_ALL_LIKE_NAME = "Customer.FindAllLikeName";
	static final String LOAD_CUSTOMER_CONTACT_REPORT = "Customer.LoadCustomerContactReport";
	/**
	 * Stored procedure name constants
	 */
	/**
	 * Factory method to create an empty <code>Customer</code> entity with an already assigned identifier.
	 * The advantage of this approach, coupled with the fact that we're using the id for equals,
	 * is that equality test's are immediately possible and therefore independent from the persistence state
	 * of the entity. This in contrast to other solutions where auto generated Keys are used
	 * which must wait until the object is persisted or differentiate between transient and managed states.
	 * @return a new (transient), empty <code>Customer</code> entity with an already assigned identifier.
	 */
	Customer create();
	/**
	 * Factory method to create an empty <code>Customer</code> taking all not null attributes.
	 *
	 * @param firstName
	 * @param lastName
	 * @param emailAddress
	 * @param birthDate
	 * @param invoiceAddress
	 * @return a new (transient ) <code>Customer</code> entity with the given natural identifier and not null attributes set.
	 */
	Customer create(String firstName,String lastName,String emailAddress,Date birthDate,Address invoiceAddress);
	/**
	 * Factory method to create and save an empty <code>Customer</code> taking all not null attributes.
	 *
	 * @param firstName
	 * @param lastName
	 * @param emailAddress
	 * @param birthDate
	 * @param invoiceAddress
	 * @return a new (transient) <code>Customer</code> entity with the given natural identifier and not null attributes set.
	 */
	Customer createAndSave(String firstName,String lastName,String emailAddress,Date birthDate,Address invoiceAddress);
	/**
	 *
	 * @return a collection containing 0 or more persistent Customer instances
	 */
	Collection<Customer> findAllCustomers();
	/**
	 *
	 * @param firstName
	 * @param lastName
	 * @return a collection containing 0 or more persistent Customer instances
	 */
	Collection<Customer> findAllLikeName(String firstName,String lastName);
	/**
	 *
	 * @return a collection containing 0 or more persistent CustomerContactReport instances
	 */
	Collection<CustomerContactReport> loadCustomerContactReport();
}
