package org.openxma.demo.customer.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import org.openxma.demo.customer.Address;
import org.openxma.demo.customer.dao.CustomerDao;
import org.openxma.demo.customer.dto.CustomerContactReport;
import org.openxma.demo.customer.model.Customer;
import org.openxma.demo.customer.model.impl.CustomerImpl;
/**
 * @see CustomerDao
 */
@SuppressWarnings("unused")
public abstract class CustomerDaoGenImpl extends GenericDaoHibernateImpl<Customer,String> implements CustomerDao {
	
	public CustomerDaoGenImpl() {
		super(CustomerImpl.class);
	}
	
	/**
	 * @param context the context for the object creation
	 * @return a transient instance of entity <code>Customer</code> with an already assigned identifier.
	 */
	public Customer createEntity(Object context) {
		return create();
	}
	
	public Customer create() 
	{
		Customer customer = createEntityInstance();
		customer.setOid(dataFieldMaxValueIncrementer.nextStringValue());
		return customer;
	}
	
	public Customer create(String firstName,String lastName,String emailAddress,Date birthDate,Address invoiceAddress) {
		Assert.notNull(firstName, "Parameter 'firstName' must not be null");
		Assert.notNull(lastName, "Parameter 'lastName' must not be null");
		Assert.notNull(emailAddress, "Parameter 'emailAddress' must not be null");
		Assert.notNull(birthDate, "Parameter 'birthDate' must not be null");
		Assert.notNull(invoiceAddress, "Parameter 'invoiceAddress' must not be null");
		Customer customer = (Customer) create();
		customer.setFirstName(firstName);
		customer.setLastName(lastName);
		customer.setEmailAddress(emailAddress);
		customer.setBirthDate(birthDate);
		customer.setInvoiceAddress(invoiceAddress);
		return customer;
	}
	
	public Customer createAndSave(String firstName,String lastName,String emailAddress,Date birthDate,Address invoiceAddress) {
		Customer customer = create(firstName,lastName,emailAddress,birthDate,invoiceAddress);
		saveOrUpdate(customer);
		return customer;
	}
	
	public Collection<Customer> findAllCustomers() {
		String queryName = CustomerDao.FIND_ALL_CUSTOMERS;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyFindAllCustomersQueryHints(namedQuery);
		return list(namedQuery);
	}
	
	protected void applyFindAllCustomersQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<Customer> findAllLikeName(String firstName,String lastName) {
		String queryName = CustomerDao.FIND_ALL_LIKE_NAME;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		namedQuery.setParameter("firstName", firstName);
		namedQuery.setParameter("lastName", lastName);
		applyFindAllLikeNameQueryHints(namedQuery,firstName,lastName);
		return list(namedQuery);
	}
	
	protected void applyFindAllLikeNameQueryHints(Query query,String firstName,String lastName) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	public Collection<CustomerContactReport> loadCustomerContactReport() {
		String queryName = CustomerDao.LOAD_CUSTOMER_CONTACT_REPORT;
		Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery(queryName);
		applyLoadCustomerContactReportQueryHints(namedQuery);
		return getNamedQueryMapper(queryName).createAndMapMany(list(namedQuery), CustomerContactReport.class, Collections.singletonMap(QUERY_NAME, queryName));
	}
	
	protected void applyLoadCustomerContactReportQueryHints(Query query) {
		// override this method and apply other hints that influence how a query is executed
	}
	
	protected CustomerContactReport mapLoadCustomerContactReportTuple(Object[] tuple, int rowNumber) {
		CustomerContactReport customerContactReport = null;
		if (tuple!=null) {
			customerContactReport = new CustomerContactReport();
			customerContactReport.setFirstName((String)tuple[0]);
			customerContactReport.setLastName((String)tuple[1]);
			customerContactReport.setEmailAddress((String)tuple[2]);
		}
		return customerContactReport;
	}
	
	protected Mapper getNamedQueryMapper(String queryName) {
		return new CustomerDaoQueryMapper();
	}
	
	protected class CustomerDaoQueryMapper extends NamedQueryMapper {
		
	@SuppressWarnings("unchecked")
	public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
		if ("Customer.LoadCustomerContactReport".equals(getQueryName(context))) {
			return (E) mapLoadCustomerContactReportTuple((Object[]) sourceObject, 1);
		}
		return super.mapOne(sourceObject, targetObject);
	}
}
}
