package org.openxma.demo.customer.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import java.util.Collection;
import org.openxma.demo.customer.dao.CustomerDao;
import org.openxma.demo.customer.dto.CustomerContactReport;
import org.openxma.demo.customer.dto.CustomerView;
import org.openxma.demo.customer.model.Customer;
import org.openxma.demo.customer.service.CustomerDas;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class CustomerDasGenImpl implements CustomerDas {
	protected final Log logger = LogFactory.getLog(getClass());
	@Autowired(required=false)
	protected Mapper mapper;
	protected CustomerDao customerDao;
	
	@Autowired
	public void setCustomerDao(CustomerDao customerDao) {
		this.customerDao = customerDao;
	}
	
	@Transactional
	public CustomerView save(CustomerView customerView) {
		Assert.notNull(customerView, "parameter 'customerView' must not be null");
		Customer customer = this.mapper.createAndMapOne(customerView, Customer.class,"saveCustomer");
		this.customerDao.saveOrUpdate(customer);
		return this.mapper.mapOne(customer,new CustomerView());
	}
	
	@Transactional(readOnly = true)
	public CustomerView loadCustomerView(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
		return this.mapper.createAndMapOne(this.customerDao.load(oid),CustomerView.class);
	}
	
	@Transactional
	public void update(CustomerView customerView) {
		Assert.notNull(customerView, "parameter 'customerView' must not be null");
		Customer mappedCustomer = this.mapper.createAndMapOne(customerView, Customer.class);
		Customer customer = this.customerDao.load(mappedCustomer.getOid());
		if (!customer.getVersion().equals(mappedCustomer.getVersion())) {
			throw new OptimisticLockingFailureException("Entity 'Customer' was updated or deleted by another transaction.");
		}
		this.mapper.mapOne(customerView, customer,"updateCustomer");
	}
	
	@Transactional
	public void deleteCustomer(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
		this.customerDao.delete(oid);
	}
	
	@Transactional
	public CustomerView saveOrUpdate(CustomerView customerView) {
		Assert.notNull(customerView, "parameter 'customerView' must not be null");
		if ( customerView.getVersion() == null ) {
			return this.save(customerView);
		}
		else {
			this.update(customerView);
			return customerView;
		}
	}
	
	
	@Transactional(readOnly=true)
	public Collection<CustomerView> findAllCustomers() {
		return this.mapper.createAndMapMany(this.customerDao.findAllCustomers(),CustomerView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<CustomerView> findAllLikeName(String firstName,String lastName) {
		return this.mapper.createAndMapMany(this.customerDao.findAllLikeName(firstName,lastName),CustomerView.class);
	}
	
	@Transactional(readOnly=true)
	public Collection<CustomerContactReport> loadCustomerContactReport() {
		return this.customerDao.loadCustomerContactReport();
	}
	
}
