package org.openxma.demo.customer.validation;

import org.openxma.demo.customer.model.Customer;
import org.openxma.dsl.platform.validation.Validators;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import static org.openxma.dsl.platform.validation.ValidatorsHelper.*;


@SuppressWarnings("boxing")
public abstract class CustomerGenValidator extends Validators {
	
	@SuppressWarnings("unchecked")
	public boolean supports(Class clazz) {
		return Customer.class.isAssignableFrom(clazz);
	}
	
	public Errors validate(Customer customer) {
		Errors errors = createErrors(customer);
		validate(customer,errors);
		return errors;
	}
	
	public void validate(Object object, Errors errors) {
		Customer customer = (Customer) object;
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "firstName", "field.required");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "lastName", "field.required");
		ValidationUtils.rejectIfEmptyOrWhitespace(errors, "emailAddress", "field.required");
		ValidationUtils.rejectIfEmpty(errors, "birthDate", "field.required");
		ValidationUtils.rejectIfEmpty(errors, "invoiceAddress", "field.required");
		rejectIfMaxLength(errors,"firstName",customer.getFirstName(),25);
		rejectIfMaxLength(errors,"lastName",customer.getLastName(),25);
		rejectIfMaxLength(errors,"emailAddress",customer.getEmailAddress(),40);
		rejectIfMaxIntegerDigits(errors,"buyLimit",customer.getBuyLimit(),18);
		rejectIfMaxFractionDigits(errors,"buyLimit",customer.getBuyLimit(),2);
	}
}
