namespace org.openxma.demo.customer.^xma.jsfcomponent
import org.openxma.demo.customer.*
 
component xma::JsfComponent {

	data Customer customer
	command testComponentCommand
	
	page CSearchPage "Customer Search" uses CustomerDas {
		property String myPageProperty
		property Integer myPageProperty1
		
		data Customer searchCustomer
		data Customer[] foundCustomers

		command findCustomer
		command initGui
		command selectCustomer
		command editDetails

		eventmapping {
			editButton -> editDetails
			onInit -> initGui, findCustomer
			findButton -> findCustomer
			customerTable.onSelection -> selectCustomer
		}
		
		panel sForm {
			vpanel customerTableHeader style=customizableStyle {
				searchCustomer.firstName
				searchCustomer.lastName & button findButton "Find" style=primaryStyle
			}
			table customerTable key=foundCustomers.oid top=customerTableHeader bottom=100% pageable=true 
			{
				foundCustomers.firstName width=80
				foundCustomers.birthDate width=80
				foundCustomers.lastName width=80
				foundCustomers.emailAddress width=80
				column manualColumn label="columnLabel" 
			}
			button editButton "Edit details" /* enabled=customerTable.isSelected() */
			seperator sep 
			tabfolder tabFolder {
				tabpage firstTab label="Name" {
					customer.firstName
					customer.lastName
				}
				tabpage secondTab label="Other" {
					customer.birthDate
					customer.emailAddress
					label "Other text: "
					text myText2
				}
			}
		}
	}
	
	page CDetailPage "Customer Details" uses CustomerDas {
		property String customerId
		property Boolean edit
		
		command initGui
		command saveOrUpdate
		command invokeSearchPage
		
		eventmapping {
			onInit -> initGui
			saveButton -> saveOrUpdate 
			backButton -> invokeSearchPage
		}
		
		panel sForm {
			vpanel customerDetails {
				customer.firstName 
				customer.lastName 
				customer.birthDate
				customer.emailAddress 
				customer.category
				customer.vip
				customer.buyLimit 
				button saveButton "Save/Update" style=primaryStyle
				button backButton "Back"
			}
		}
	}
	
	page CustomizeFaceletPage "Facelet Customization" {
		label "With style you can mark sections for override in manual facelet per ui:insert tag."
		panel mypanel style=customizableStyle {}
	}
	
	page ParentWindow "Parent Window" {
		uicommand invokeChildDialog : invoke ChildDialog 
		eventmapping {
			showDialogButton -> invokeChildDialog
		}
		panel pp {
			label "This is the parent window. First citizen browser window."
			button showDialogButton "Show dialog" style=warningStyle
		}
	}
	
	page ChildDialog "Child Dialog" {
		eventmapping {
			okButton -> closeCancel			
		}
		panel p {
			label "This is a child dialog. It's not a browser window: It is faked with HTML/JS construction."
			button okButton "OK" style=successStyle
		}
	}
	
	page RemoteCallsPage "Remote Calls" {
		command myServerMethodDirect
		command myServerMethod
		uicommand myClientMethod
		
		eventmapping {
			myButtonClient -> myClientMethod			
			myButtonServer -> myServerMethodDirect			
		}
		panel p {
			label "You can call xma server commands from within client JS. Important: Make authorization check in Service and/or Presentation layer!"
			button myButtonServer "Remote Call - Direct Ajax Call" style=primaryStyle
			button myButtonClient "Remote Call - Client JavaScript"
		}
	}
	
	page ChartsPage "HTML 5 Charts" {
		label "Easy integration of HTML 5 charts"
		panel chartsPanel style=customizableStyle {}
	}
	
	page ValidationPage "JSF 303 Validation - Required Fields" {
		data Customer cus
		command sendForm
		
		eventmapping {
			sendFormButton -> sendForm
		}
		
		// new customer with not all required fields
		panel p {
			cus.firstName
			button sendFormButton "send Form" style=primaryStyle
		}
	}
	
	page InvokeComponentPage "Invoke component" {
		uicommand invokePresentationComponent : invoke ^xma.presentation.Presentation
		eventmapping {
			myButton -> invokePresentationComponent
		}
		panel p3 {
			label "Invoke a component from within another component: "
			button myButton "Invoke component" style=primaryStyle
			
		}
	}
	
	page GoogleMapsPage "Google Maps" {
		label "Easy integration of Google Maps"
		panel mapsPanel style=customizableStyle {}
	}
	
	page WidgetsPage "Other widgets" {
		command init
		command send
		eventmapping {
			onInit -> init
			sendButton -> send
		}
		
		vpanel {
			label "Text: " | text mytext
			label "Disabled Text: " | text mydisabledtext enabled=false 
			label "Combo box: " | combo mycombo
			label "Disabled Combo box: " | combo mydisabledcombo enabled=false
			label "Checkbox: " | checkbox mycheck "checked"
			label "Disabled Checkbox: " | checkbox mydisabledcheck "checked" enabled=false
			label "Listbox: " | listbox mylistbox
			label "Disabled Listbox: " | listbox mydisabledlistbox enabled=false
			label "Tree:" | tree mytree
		} 
		
		seperator myseparator		
		
		button sendButton "send" style=primaryStyle
		button disabledButton "disabled" enabled=false
		
	}
  
  page BootstrapSemanticButtonStyles "Twitter Bootstrap Semantic Button Styles" {
    hpanel {
      label "Buttons"     
      button buttonNormal "Nomal"
      button buttonCloseok "Success" style=successStyle
      button buttonAcknowledge "Info" style=infoStyle
      button buttonWarning "Warning" style=warningStyle
      button buttonConfirmCritical "Danger" style=dangerStyle
      button mainButton "Primary" style=primaryStyle 
    }
  
  }
  
  page ConditionsPage "Conditions" {
  	property Boolean edit
  	command enable 
  	command disable 
  	eventmapping {
  		enableButton 	-> enable
  		disableButton 	-> disable
  	}
  	conditions {
  		editMode = edit == true
  		viewMode = edit == false
  	} 
  	hpanel {
	  	button enableButton "enable" enabled=viewMode
  		button disableButton "disable" enabled=editMode
  	}
  }
  page ExpressionsPage "Expressions" {
  	property Boolean edit
  	command enable 
  	command disable 
  	eventmapping {
  		enableButton 	-> enable
  		disableButton 	-> disable
  	}
  	hpanel {
	  	button enableButton "enable" enabled=edit==false
  		button disableButton "disable" enabled=edit
  		
  	}
  }
  page ExpressionsStringsPage "Expressions/Strings" {
  	property String mode
  	command setViewMode 
  	command setEditMode
  	eventmapping {
  		setViewModeButton 	-> setViewMode
  		setEditModeButton 	-> setEditMode
  	}
  	hpanel {
	  	button setViewModeButton "View mode" enabled=mode=='edit'
  		button setEditModeButton "Edit mode" enabled=mode!='edit'
  		
  	}
  }
  
  page TableCustomizerPage "Customizing Tables" uses CustomerDas {
  	data Customer[] customers
  	command init
  	eventmapping {
  		onInit		-> init
  	}
  	
  	panel tablePanel style=customizableStyle {
		table customerTable key=customers.oid pageable=true 
		{
			customers.firstName width=80
			customers.birthDate width=80
			customers.lastName width=80
			customers.emailAddress width=80
		}
  	}
  }
	
}

style customizableStyle {
	customization: facelet
}
style successStyle {
	widget-style: style-success
}
style warningStyle {
	widget-style: style-warning
}
style dangerStyle {
	widget-style: style-danger
}
style primaryStyle {
	widget-style: style-primary
}
style infoStyle {
	widget-style: style-info
}